(*
 * Copyright 2005 Toshiyuki Maeda
 *
 * This file is part of TALK.
 *
 * TALK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * TALK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)

open Types
open Output_types

(************************************
        Type cast functions
 ************************************)

type error_type =
	| Cast_err_code of code_type
	| Cast_err_concrete of concrete_type
	| Cast_err_type of typ
	| Cast_err_array of array_type
	| Cast_err_heap of heap_type

exception Error of error_type

let raise_error_code_type ct =
	raise (Error (Cast_err_code ct))
let raise_error_concrete_type ct =
	raise (Error (Cast_err_concrete ct))
let raise_error_type t =
	raise (Error (Cast_err_type t))
let raise_error_array_type at =
	raise (Error (Cast_err_array at))
let raise_error_heap_type ht =
	raise (Error (Cast_err_heap ht))

let code_to_state ct =
	match ct with
        | State state ->
                state
        | _ ->
		raise_error_code_type ct

let code_to_named_type ct =
	match ct with
	| NameCode nt ->
		nt
	| _ ->
		raise_error_code_type ct

let concrete_to_package ct =
        match ct with
        | Pack pt ->
                pt
        | _ ->
		raise_error_concrete_type ct

let concrete_to_tuple ct =
	match ct with
	| Tuple tt ->
		tt
	| _ ->
		raise_error_concrete_type ct

let typ_to_concrete typ =
	match typ with
        | Concrete ct ->
                ct
        | _ ->
		raise_error_type typ

let typ_to_named_type typ =
	match typ with
        | Name (name, cs) ->
                (name, cs)
        | _ ->
		raise_error_type typ

open Integer_op

let array_type_to_typ str ics at =
	let len = at.at_len in

	let ic = (ie_inte len) =.. (ie_int 1) in
(*
	if (Constraint_solver.implies str ics [ic]) then
*)
	if (Constraint_solver.implies_one ics ic) then
		at.at_type
	else
		raise_error_array_type at

let heap_to_array ht =
	match ht with
	| Array at ->
		at
	| _ ->
		raise_error_heap_type ht

let heap_to_rich_stack ht =
	match ht with
	| Stack rstk ->
		rstk
	| _ ->
		raise_error_heap_type ht
