(*
 * Copyright 2005 Toshiyuki Maeda
 *
 * This file is part of TALK.
 *
 * TALK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * TALK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)

open Types
open Annotations
open Instructions

type debug_info = string list

type p_operand =
        | P_Op_Mem of mem_operand * address
        | P_Op_Imd of Big_int.big_int * operand_size
        | P_Op_Reg of register

type pseudo_inst =
	| P_ROLL of integer * named_type
	| P_UNROLL of integer
	| P_PACK of integer * pack_type * constructor list
	| P_UNPACK of debug_info * integer
	| P_FALL
	| P_SPLIT of debug_info * integer * integer
	| P_CONCAT of debug_info * integer * integer
	| P_TUPLE_SPLIT of debug_info * integer * integer
	| P_TUPLE_CONCAT of integer * integer

	| P_CALLN	of p_operand
	| P_JMPN	of p_operand
	| P_RET
	| P_PUSH	of debug_info * p_operand
	| P_PUSHF	of debug_info
	| P_MOV		of p_operand * p_operand
	| P_ADD		of debug_info * p_operand * p_operand
	| P_SUB		of debug_info * p_operand * p_operand
	| P_CMP		of debug_info * p_operand * p_operand
	| P_DEC		of debug_info * p_operand
	| P_IMUL	of debug_info * p_operand * p_operand * p_operand
	| P_POP		of debug_info * p_operand
	| P_POPF	of debug_info
	| P_Jcc		of condition_code * p_operand

	(* These are auxiliary instructions that can be represented with above instructions. *)
	| P_CAST_TO_TUPLE of debug_info * integer
	| P_CAST_TO_ARRAY of debug_info * integer
	| P_CAST_TO_STACK of debug_info * integer

type pseudo_insts =
{
        pis_before      : pseudo_inst list;
        pis_after       : pseudo_inst list;
}

let get_dname = Util.pop_with_def "__"
