(*
 * Copyright 2005 Toshiyuki Maeda
 *
 * This file is part of TALK.
 *
 * TALK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * TALK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)

open Types
open Typing_common
open Output_types

open Machine
open Util

exception Type_error of string

let raise_type_error str = raise (Type_error str)
let failwithf_type_error f = failwithf_exn raise_type_error f

exception Typechecker_bug of string
let raise_typechecker_bug str = raise (Typechecker_bug str)
let failwithf_typechecker_bug f = failwithf_exn raise_typechecker_bug f

let error_tuple_size_mismatch size1 size2 =
        failwithf_type_error "Tuple size mismatch: %d <===> %d" size1 size2

let error_unbound_type_variable state tv =
        failwithf_type_error "Unbound type variable %a (maybe renamed...)" output_tv (state, tv)

let error_at_typing_block mesg blk =
        failwithf_type_error "@[[block 0x%08nx]@ %s@]" blk.blk_segment.seg_va mesg

let error_at_typing_code_block mesg c_blk =
        failwithf_type_error "@[[block 0x%08nx]@ %s@]" c_blk.cblk_segment.seg_va mesg

let error_stack_not_null sz =
        failwithf_type_error "Stack is not null (size = %d)" sz

let error_stack_too_short sz =
        failwithf_type_error "Stack is too short (need more %d words)" sz

let error_integer_mismatch bi1 n2 =
        failwithf_type_error "Integer mismatch 0x%a and 0x%08nx" Big_int_util.output_big_int_x bi1 n2

let error_type_variables_not_instantiated kinds =
        failwithf_type_error "There are non-instantiated type variables: %d" (List.length kinds)

let error_incompatible_any outputter state a =
        failwithf_type_error "Incompatible type: %a" outputter (state, a)

let error_incompatible_type =
	error_incompatible_any output_type

let error_incompatible_concrete_type =
	error_incompatible_any output_ct
